#include "config.h"

#include "abc-utils.h"
#include <json-c/json.h>

int check_container_checksum(const char **args, arg_unused int n_args)
{
	const char *container_name = args[0];
	const char *container_checksum = args[1];

	autofreejson struct json_object *container_info =
		find_container_info_by_name(container_name);
	if (container_info == NULL) {
		error("Container named '%s' not found\n", container_name);
		return 1;
	}

	const char *container_id = container_info_get_id(container_info);
	if (container_id == NULL) {
		error("Unable to get container id\n");
		return 1;
	}

	autofreejson struct json_object *config =
		load_container_config(container_id);
	if (config == NULL) {
		error("Container id '%s' not found\n", container_id);
		return 1;
	}

	canonicalize_container_config(config, container_id);

	const char *js = json_object_to_json_string(config);
	char digest_hex[SHA256_DIGEST_HEX_LEN + 1];
	sha256_hex((uint8_t *)js, strlen(js), digest_hex);

	if (strcmp(container_checksum, digest_hex) != 0) {
		error("Container '%s' config checksum was '%s', but expected '%s'\n",
		      container_name, digest_hex, container_checksum);
		return 1;
	}

	return 0;
}
