#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <json-c/json.h>

#define N_ELEMENTS(arr) (sizeof(arr) / sizeof((arr)[0]))

#define arg_unused __attribute__((unused))

#define autofree __attribute__((cleanup(cleanup_free)))
#define autoclose __attribute__((cleanup(cleanup_close)))
#define autoclosedir __attribute__((cleanup(cleanup_closedir)))
#define autofreejson __attribute__((cleanup(cleanup_json)))

extern bool opt_verbose;

extern void *xmalloc(size_t size) __attribute__((__malloc__))
__attribute__((__alloc_size__(1)));
extern void *xcalloc(size_t nmemb, size_t size)
	__attribute__((__alloc_size__(1, 2)));
extern void *xrealloc(void *ptr, size_t size) __attribute__((warn_unused_result));

static inline void cleanup_free(void *p)
{
	free(*(void **)p);
}

static inline void cleanup_close(const int *fd)
{
	if (*fd >= 0)
		close(*fd);
}

static inline void cleanup_closedir(DIR **dir)
{
	if (*dir)
		closedir(*dir);
}

static inline void cleanup_json(void *p)
{
	void **pp = (void **)p;

	if (*pp != NULL)
		json_object_put(*pp);
}

static inline void *steal_ptr(void *_pp)
{
	void **pp = (void **)_pp;
	void *p = NULL;

	p = *pp;
	*pp = NULL;

	return p;
}

extern void verbose(const char *format, ...)
	__attribute__((__format__(printf, 1, 2)));
extern void error(const char *format, ...)
	__attribute__((__format__(printf, 1, 2)));
extern void fatal(const char *format, ...) __attribute__((__noreturn__))
__attribute__((__format__(printf, 1, 2)));
extern void oom(void) __attribute__((__noreturn__));

extern bool str_has_prefix(const char *str, const char *prefix);
extern bool str_has_suffix(const char *str, const char *suffix);
extern char *join_path(const char *a, const char *b);
extern char *read_file(const char *filename);
char **str_split(char *row, char split_at, size_t *size);

#define SHA256_DIGEST_LEN 32
#define SHA256_DIGEST_HEX_LEN (SHA256_DIGEST_LEN * 2)
#define SHA256_DATASIZE 64
typedef struct {
	uint32_t buf[8];
	uint32_t bits[2];

	uint8_t data[SHA256_DATASIZE];
	uint8_t digest[SHA256_DIGEST_LEN];
} Sha256sum;

void sha256_sum_init(Sha256sum *sha256);
void sha256_sum_update(Sha256sum *sha256, const uint8_t *buffer, size_t length);
void sha256_sum_close(Sha256sum *sha256);
void sha256_digest_to_hex(const uint8_t *csum, char *buf);

void sha256_hex(const uint8_t *buffer, size_t length, char *hex_buf);

struct json_object *find_container_info_by_name(const char *container_name);
const char *container_info_get_id(struct json_object *container_info);
struct json_object *load_container_config(const char *container_id);
void canonicalize_container_config(struct json_object *config,
				   const char *container_id);
